@@ -39,7 +39,7 @@ class obj_traits:
     @staticmethod
     def is_experimental(obj):
         try:
-            return True if re.search("Exp$", obj['name']) else False
+            return True if re.search("Exp[0-9]*$", obj['name']) else False
         except:
             return False
 
@@ -1937,4 +1937,4 @@ Public:
 """
 def isNewProcTable(tableName):
 
-    return tableName in ["zesGetGlobalProcAddrTable","zesGetOverclockProcAddrTable"]
\ No newline at end of file
+    return tableName in ["zesGetGlobalProcAddrTable","zesGetOverclockProcAddrTable","zesGetVFManagementProcAddrTable"]
\ No newline at end of file
