@@ -23,6 +23,7 @@
 typedef struct _zel_global_callbacks_t
 {
     ze_pfnInitCb_t                                                  pfnInitCb;
+    ze_pfnInitDriversCb_t                                           pfnInitDriversCb;
 } zel_global_callbacks_t;
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -142,6 +143,8 @@ typedef struct _zel_command_list_callbacks_t
     ze_pfnCommandListAppendLaunchCooperativeKernelCb_t              pfnAppendLaunchCooperativeKernelCb;
     ze_pfnCommandListAppendLaunchKernelIndirectCb_t                 pfnAppendLaunchKernelIndirectCb;
     ze_pfnCommandListAppendLaunchMultipleKernelsIndirectCb_t        pfnAppendLaunchMultipleKernelsIndirectCb;
+    ze_pfnCommandListGetNextCommandIdWithKernelsExpCb_t             pfnGetNextCommandIdWithKernelsExpCb;
+    ze_pfnCommandListUpdateMutableCommandKernelsExpCb_t             pfnUpdateMutableCommandKernelsExpCb;
     ze_pfnCommandListAppendImageCopyToMemoryExtCb_t                 pfnAppendImageCopyToMemoryExtCb;
     ze_pfnCommandListAppendImageCopyFromMemoryExtCb_t               pfnAppendImageCopyFromMemoryExtCb;
     ze_pfnCommandListHostSynchronizeCb_t                            pfnHostSynchronizeCb;
@@ -232,6 +235,7 @@ typedef struct _zel_kernel_callbacks_t
     ze_pfnKernelGetPropertiesCb_t                                   pfnGetPropertiesCb;
     ze_pfnKernelGetNameCb_t                                         pfnGetNameCb;
     ze_pfnKernelSetGlobalOffsetExpCb_t                              pfnSetGlobalOffsetExpCb;
+    ze_pfnKernelGetBinaryExpCb_t                                    pfnGetBinaryExpCb;
     ze_pfnKernelSchedulingHintExpCb_t                               pfnSchedulingHintExpCb;
 } zel_kernel_callbacks_t;
 
