@@ -124,14 +124,42 @@ namespace loader
         }
     }
 
-    ze_result_t context_t::check_drivers(ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly) {
-        if (debugTraceEnabled) {
-            std::string message = "check_drivers(" + std::string("flags=") + loader::to_string(flags) + ")";
-            debug_trace_message(message, "");
+    std::string to_string(const ze_init_driver_type_desc_t *desc) {
+        std::string flags_value;
+        if (desc->flags & ZE_INIT_DRIVER_TYPE_FLAG_GPU) {
+            flags_value+= "|ZE_INIT_DRIVER_TYPE_FLAG_GPU|";
         }
+        if (desc->flags & ZE_INIT_DRIVER_TYPE_FLAG_NPU) {
+            flags_value+= "|ZE_INIT_DRIVER_TYPE_FLAG_NPU|";
+        }
+        return flags_value;
+    }
+
+    ze_result_t context_t::check_drivers(ze_init_flags_t flags, ze_init_driver_type_desc_t* desc, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly) {
+        if (debugTraceEnabled) {
+            if (desc) {
+                std::string message = "check_drivers(" + std::string("desc->flags=") + loader::to_string(desc) + ")";
+                debug_trace_message(message, "");
+            } else {
+                std::string message = "check_drivers(" + std::string("flags=") + loader::to_string(flags) + ")";
+                debug_trace_message(message, "");
+            }
+        }
+        // If zeInitDrivers is not supported by this driver, but zeInitDrivers is called first, then return uninitialized.
+        if (desc && !loader::context->initDriversSupport) {
+            std::string message = "zeInitDrivers called first, but not supported by driver, returning uninitialized.";
+            debug_trace_message(message, "");
+            return ZE_RESULT_ERROR_UNINITIALIZED;
+        }
+
+
         bool return_first_driver_result=false;
         std::string initName = "zeInit";
         driver_vector_t *drivers = &zeDrivers;
+        // If desc is set, then this is zeInitDrivers.
+        if (desc) {
+            initName = "zeInitDrivers";
+        }
         // If this is sysmanOnly check_drivers, then zesInit is being called and we need to use zesDrivers.
         if (sysmanOnly) {
             drivers = &zesDrivers;
@@ -144,22 +172,29 @@ namespace loader
         for(auto it = drivers->begin(); it != drivers->end(); )
         {
             std::string freeLibraryErrorValue;
-            ze_result_t result = init_driver(*it, flags, globalInitStored, sysmanGlobalInitStored, sysmanOnly);
+            ze_result_t result = init_driver(*it, flags, desc, globalInitStored, sysmanGlobalInitStored, sysmanOnly);
             if(result != ZE_RESULT_SUCCESS) {
-                if (debugTraceEnabled) {
-                    std::string errorMessage = "Check Drivers Failed on " + it->name + " , driver will be removed. " + initName + " failed with ";
-                    debug_trace_message(errorMessage, loader::to_string(result));
-                }
-                it = drivers->erase(it);
-                // If the number of drivers is now ==1, then we need to reinit the ddi tables to pass through.
-                // If ZE_ENABLE_LOADER_INTERCEPT is set to 1, then even if drivers were removed, don't reinit the ddi tables.
-                if (drivers->size() == 1 && !loader::context->forceIntercept) {
-                    *requireDdiReinit = true;
+                // If the driver has already been init and handles are to be read, then this driver cannot be removed from the list.
+                // Also, if any driver supports zeInitDrivers, then no driver can be removed to allow for different sets of drivers.
+                if (!it->driverInuse && !loader::context->initDriversSupport) {
+                    if (debugTraceEnabled) {
+                        std::string errorMessage = "Check Drivers Failed on " + it->name + " , driver will be removed. " + initName + " failed with ";
+                        debug_trace_message(errorMessage, loader::to_string(result));
+                    }
+                    it = drivers->erase(it);
+                    // If the number of drivers is now ==1, then we need to reinit the ddi tables to pass through.
+                    // If ZE_ENABLE_LOADER_INTERCEPT is set to 1, then even if drivers were removed, don't reinit the ddi tables.
+                    if (drivers->size() == 1 && !loader::context->forceIntercept) {
+                        *requireDdiReinit = true;
+                    }
+                } else {
+                    it++;
                 }
                 if(return_first_driver_result)
                     return result;
-            }
-            else {
+            } else {
+                // If this is a single driver system, then the first success for this driver needs to be set.
+                it->driverInuse = true;
                 it++;
             }
         }
@@ -170,7 +205,7 @@ namespace loader
         return ZE_RESULT_SUCCESS;
     }
 
-    ze_result_t context_t::init_driver(driver_t &driver, ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool sysmanOnly) {
+    ze_result_t context_t::init_driver(driver_t &driver, ze_init_flags_t flags, ze_init_driver_type_desc_t* desc, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool sysmanOnly) {
 
         if (sysmanOnly) {
             auto getTable = reinterpret_cast<zes_pfnGetGlobalProcAddrTable_t>(
@@ -233,34 +268,66 @@ namespace loader
                 return ZE_RESULT_ERROR_UNINITIALIZED;
             }
 
-            if(nullptr == global.pfnInit) {
-                if (debugTraceEnabled) {
-                    std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
-                    debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+            if (!desc) {
+                if(nullptr == global.pfnInit) {
+                    if (debugTraceEnabled) {
+                        std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
+                        debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+                    }
+                    return ZE_RESULT_ERROR_UNINITIALIZED;
                 }
-                return ZE_RESULT_ERROR_UNINITIALIZED;
-            }
 
-            auto pfnInit = global.pfnInit;
-            if(nullptr == pfnInit || globalInitStored->pfnInit == nullptr) {
-                if (debugTraceEnabled) {
-                    std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
-                    debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+                auto pfnInit = global.pfnInit;
+                if(nullptr == pfnInit || globalInitStored->pfnInit == nullptr) {
+                    if (debugTraceEnabled) {
+                        std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
+                        debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+                    }
+                    return ZE_RESULT_ERROR_UNINITIALIZED;
                 }
-                return ZE_RESULT_ERROR_UNINITIALIZED;
-            }
 
-            // Use the previously init ddi table pointer to zeInit to allow for intercept of the zeInit calls
-            ze_result_t res = globalInitStored->pfnInit(flags);
-            // Verify that this driver successfully init in the call above.
-            if (driver.initStatus != ZE_RESULT_SUCCESS) {
-                res = driver.initStatus;
+                // Use the previously init ddi table pointer to zeInit to allow for intercept of the zeInit calls
+                ze_result_t res = globalInitStored->pfnInit(flags);
+                // Verify that this driver successfully init in the call above.
+                if (driver.initStatus != ZE_RESULT_SUCCESS) {
+                    res = driver.initStatus;
+                }
+                if (debugTraceEnabled) {
+                    std::string message = "init driver " + driver.name + " zeInit(" + loader::to_string(flags) + ") returning ";
+                    debug_trace_message(message, loader::to_string(res));
+                }
+                return res;
+            } else {
+                if(nullptr == global.pfnInitDrivers) {
+                    if (debugTraceEnabled) {
+                        std::string errorMessage = "init driver " + driver.name + " failed, zeInitDrivers function pointer null. Returning ";
+                        debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+                    }
+                    return ZE_RESULT_ERROR_UNINITIALIZED;
+                }
+
+                auto pfnInitDrivers = global.pfnInitDrivers;
+                if(nullptr == pfnInitDrivers || globalInitStored->pfnInitDrivers == nullptr) {
+                    if (debugTraceEnabled) {
+                        std::string errorMessage = "init driver " + driver.name + " failed, pfnInitDrivers function pointer null. Returning ";
+                        debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+                    }
+                    return ZE_RESULT_ERROR_UNINITIALIZED;
+                }
+
+                // Use the previously init ddi table pointer to zeInit to allow for intercept of the zeInit calls
+                uint32_t pCount = 0;
+                ze_result_t res = globalInitStored->pfnInitDrivers(&pCount, nullptr, desc);
+                // Verify that this driver successfully init in the call above.
+                if (driver.initDriversStatus != ZE_RESULT_SUCCESS) {
+                    res = driver.initDriversStatus;
+                }
+                if (debugTraceEnabled) {
+                    std::string message = "init driver " + driver.name + " zeInitDrivers(" + loader::to_string(desc) + ") returning ";
+                    debug_trace_message(message, loader::to_string(res));
+                }
+                return res;
             }
-            if (debugTraceEnabled) {
-                std::string message = "init driver " + driver.name + " zeInit(" + loader::to_string(flags) + ") returning ";
-                debug_trace_message(message, loader::to_string(res));
-            }
-            return res;
         }
     }
 
@@ -340,7 +407,8 @@ namespace loader
             loaderLibraryPath = readLevelZeroLoaderLibraryPath();
         }
 #endif
-        debug_trace_message("Using Loader Library Path: ", loaderLibraryPath);
+        if (debugTraceEnabled)
+            debug_trace_message("Using Loader Library Path: ", loaderLibraryPath);
 
         // To allow for two different sets of drivers to be in use between sysman and core/tools, we use and store the drivers in two vectors.
         // alldrivers stores all the drivers for cleanup when the library exits.
@@ -371,12 +439,19 @@ namespace loader
         for( auto name : discoveredDrivers )
         {
             auto handle = LOAD_DRIVER_LIBRARY( name.c_str() );
-            if (debugTraceEnabled) {
-                std::string message = "Loading Driver " + name;
-                debug_trace_message(message, "");
-            }
             if( NULL != handle )
             {
+                if (debugTraceEnabled) {
+                    std::string message = "Loading Driver " + name + " succeeded";
+#ifndef _WIN32
+                    // TODO: implement same message for windows, move dlinfo to ze_util.h as a macro
+                    struct link_map *dlinfo_map;
+                    if (dlinfo(handle, RTLD_DI_LINKMAP, &dlinfo_map) == 0) {
+                        message += " from: " + std::string(dlinfo_map->l_name);
+                    }
+#endif
+                    debug_trace_message(message, "");
+                }
                 allDrivers.emplace_back();
                 allDrivers.rbegin()->handle = handle;
                 allDrivers.rbegin()->name = name;
@@ -388,8 +463,10 @@ namespace loader
             }
         }
         if(allDrivers.size()==0){
-            std::string message = "0 Drivers Discovered";
-            debug_trace_message(message, "");
+            if (debugTraceEnabled) {
+                std::string message = "0 Drivers Discovered";
+                debug_trace_message(message, "");
+            }
             zel_logger->log_error("0 Drivers Discovered");
             return ZE_RESULT_ERROR_UNINITIALIZED;
         }
@@ -424,7 +501,8 @@ namespace loader
             tracingLayerEnabled = true;
         }
         std::string tracingLayerLibraryPath = create_library_path(MAKE_LAYER_NAME( "ze_tracing_layer" ), loaderLibraryPath.c_str());
-        debug_trace_message("Tracing Layer Library Path: ", tracingLayerLibraryPath);
+        if (debugTraceEnabled)
+            debug_trace_message("Tracing Layer Library Path: ", tracingLayerLibraryPath);
         tracingLayer = LOAD_DRIVER_LIBRARY( tracingLayerLibraryPath.c_str() );
         if(tracingLayer)
         {
