@@ -31,8 +31,10 @@ namespace loader
     {
         HMODULE handle = NULL;
         ze_result_t initStatus = ZE_RESULT_SUCCESS;
+        ze_result_t initDriversStatus = ZE_RESULT_SUCCESS;
         dditable_t dditable = {};
         std::string name;
+        bool driverInuse = false;
     };
 
     using driver_vector_t = std::vector< driver_t >;
@@ -95,6 +97,8 @@ namespace loader
         zet_tracer_exp_factory_t            zet_tracer_exp_factory;
         zet_debug_session_factory_t         zet_debug_session_factory;
         zet_metric_programmable_exp_factory_t   zet_metric_programmable_exp_factory;
+        zet_metric_tracer_exp_factory_t     zet_metric_tracer_exp_factory;
+        zet_metric_decoder_exp_factory_t    zet_metric_decoder_exp_factory;
         ///////////////////////////////////////////////////////////////////////////////
         std::mutex image_handle_map_lock;
         std::mutex sampler_handle_map_lock;
@@ -112,13 +116,14 @@ namespace loader
         bool driverEnvironmentQueried = false;
 
         bool forceIntercept = false;
+        bool initDriversSupport = false;
         std::vector<zel_component_version_t> compVersions;
         const char *LOADER_COMP_NAME = "loader";
 
-        ze_result_t check_drivers(ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly);
+        ze_result_t check_drivers(ze_init_flags_t flags, ze_init_driver_type_desc_t* desc, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly);
         void debug_trace_message(std::string errorMessage, std::string errorValue);
         ze_result_t init();
-        ze_result_t init_driver(driver_t &driver, ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool sysmanOnly);
+        ze_result_t init_driver(driver_t &driver, ze_init_flags_t flags, ze_init_driver_type_desc_t* desc, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool sysmanOnly);
         void add_loader_version();
         ~context_t();
         bool intercept_enabled = false;
